use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007004
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AM
Algorithm
Batch
BigInt
DataSet
Glenn
Item
Logan
Nathan
Nick
Result
Royal
Skousen
Stanford
Theron
algorithm
analogize
bin
garfieldnate
lattice
lib
nathan
nlogan
shixilun
